from botocore.exceptions import ProfileNotFound
import subprocess
import sys


def _win_console_print(s):
    for c in s:
        msvcrt.putwch(c)
    msvcrt.putwch("\r")
    msvcrt.putwch("\n")


def _unix_console_print(s):
    with os.fdopen(os.open("/dev/tty", os.O_WRONLY | os.O_NOCTTY), "w", 1,) as tty:
        print(s, file=tty)


try:
    import msvcrt

    console_print = _win_console_print
except ImportError:
    import os

    console_print = _unix_console_print


class YubikeyTotpPrompter(object):
    def __init__(self, mfa_serial, original_prompter=None):
        self.mfa_serial = mfa_serial
        self._original_prompter = original_prompter

    def __call__(self, prompt):
        try:
            available_keys_result = subprocess.run(
                ["ykman", "oath", "list"], capture_output=True, check=True
            )
            available_keys = available_keys_result.stdout.decode("utf-8").split()
            available_keys.index(self.mfa_serial)

            console_print(
                "Generating OATH code on YubiKey. You may have to touch your YubiKey to proceed..."
            )
            ykman_result = subprocess.run(
                ["ykman", "oath", "code", "-s", self.mfa_serial], capture_output=True
            )
            console_print("Successfully created OATH code.")
            token = ykman_result.stdout.decode("utf-8").strip()
            return token
        except subprocess.CalledProcessError as e:
            print("No YubiKey found.", file=sys.stderr)
        except ValueError as e:
            print(
                "The requested mfa_serial has no OATH credentials stored on your YubiKey.",
                file=sys.stderr,
            )

        if self._original_prompter:
            return self._original_prompter(prompt)

        return None


def inject_yubikey_totp_prompter(session, **kwargs):
    try:
        providers = session.get_component("credential_provider")
    except ProfileNotFound:
        return

    config = session.get_scoped_config()
    mfa_serial = config.get("mfa_serial")
    if mfa_serial is None:
        # no MFA, so don't interfere with regular flow
        return

    assume_role_provider = providers.get_provider("assume-role")
    original_prompter = assume_role_provider._prompter
    assume_role_provider._prompter = YubikeyTotpPrompter(
        mfa_serial, original_prompter=original_prompter
    )



from .prompter import inject_yubikey_totp_prompter
from .commands import SessionEnv


def awscli_initialize(cli):
    cli.register(
        "session-initialized",
        inject_yubikey_totp_prompter,
        unique_id="inject_yubikey_totp_provider",
    )
    cli.register("building-command-table.main", awscli_register_commands)


def awscli_register_commands(command_table, session, **kwargs):
    command_table["session-env"] = SessionEnv(session)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['awscli_plugin_yubikeytotp']

package_data = \
{'': ['*']}

install_requires = \
['botocore>=1.14.14,<2.0.0']

setup_kwargs = {
    'name': 'awscli-plugin-yubikeytotp',
    'version': '1.1.0a2',
    'description': '',
    'long_description': "[![PyPI version fury.io](https://badge.fury.io/py/awscli-plugin-yubikeytotp.svg)](https://pypi.python.org/pypi/awscli-plugin-yubikeytotp/) [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n\n# Yubikey authentication for AWS CLI (and boto) made easy\n\nThis plugin enables aws-cli to directly talk to your [YubiKey](https://www.yubico.com/)\nto acquire an OATH-TOTP code using the YubiKey's CCID application.\n\nCurrently, FIDO-U2F is unsupported on both, [botocore](https://github.com/aws/aws-cli/issues/3607)\nand [aws-cli](https://github.com/aws/aws-cli/issues/3607).\nUsing aws-cli with roles and a regular OATH-TOTP token at least prompts you for\nthe TOTP code but this is quite cumbersome to use with a YubiKey.\n\n\n## Installation\n\n`awscli-plugin-yubikeytotp` can be installed from PyPI:\n```\n$ pip install awscli-plugin-yubikeytotp\n```\nIt's also possible to install it just for your user in case you don't have\npermission to install packages system-wide:\n```\n$ pip install --user awscli-plugin-yubikeytotp\n```\n\n\n### Configure AWS CLI\nTo enable the plugin, add this to your `~/.aws/config`:\n```\n[plugins]\nyubikeytotp = awscli_plugin_yubikeytotp\n```\nAlso make sure to have your MFA ARN configured for your profile:\n```\n[profile myprofile]\nrole_arn = arn:aws:iam::...\nmfa_serial = arn:aws:iam::...\nsource_profile = default\n```\n\n\n## Usage\n\nJust use the `aws` command with a custom role and the plugin will do the rest:\n```\n$ aws s3 ls --profile myprofile\nGenerating OATH code on YubiKey. You may have to touch your YubiKey to proceed...\nSuccessfully created OATH code.\n2013-07-11 17:08:50 mybucket\n2013-07-24 14:55:44 mybucket2\n```\n\n---\n**NOTE**\n\nTo AWS, the M in MFA [still stands for the number 2](https://forums.aws.amazon.com/thread.jspa?threadID=137055). Hence, if you want to log in to the AWS console with OATH-TOTP\nenabled, you will have to use `ykman oath code` to generate a key and enter it\nmanually to log in, just like with Google Authenticator or Yubikey Authenticator\nsmartphone apps.\n\nIf you want to use U2F MFA authentication for login while still being able to\nuse your YubiKey for command line, you should create multiple IAM users. Use\none user with U2F MFA to log in to the AWS console and disable access keys.\nUse the other use with Virtual MFA device and disable console login.\nOnly ever use one user in the console and the other on your command line.\n\n---\n\n\n## Tutorial\n\n### Prerequisites\n\nYou obviously have to have a YubiKey. A Yubico Security Key (the blue ones)\nare not supported as they lack the OATH application.\n\nTo setup your YubiKey, you also have to have [yubikey-manager](https://developers.yubico.com/yubikey-manager/) (or `ykman`) installed.\nYou can use it to verify that OATH is enabled for your YubiKey:\n```\n$ ykman info\nDevice type: YubiKey 5 NFC\nSerial number: 00000000\nFirmware version: 5.2.4\nForm factor: Keychain (USB-A)\nEnabled USB interfaces: OTP+FIDO+CCID\nNFC interface is enabled.\n\nApplications    USB     NFC    \nOTP             Enabled Enabled\nFIDO U2F        Enabled Enabled\nOpenPGP         Enabled Enabled\nPIV             Enabled Enabled\nOATH            Enabled Enabled\nFIDO2           Enabled Enabled\n```\n\n### Setup MFA in AWS\n\n1. Log-in to your AWS account/user and navigate to your *My Security Credentials*\npage.\n2. Take not of your *AWS account ID* at the top.\n2. Under *Multi-factor authentivation (MFA)*, click `Manage MFA device` and add\na *Virtual MFA device*.\n3. Instead of showing the QR code, click on `Show secret key` and copy the key.\n4. On a command line, run\n   ```\n   $ ykman oath add -t arn:aws:iam::${ACCOUNT_ID}:mfa/${IAM_USERNAME} ${MFA_SECRET}\n   ```\n   The strange string `arn:aws:iam::${ACCOUNT_ID}:mfa/${IAM_USERNAME}` will\n   be your user's MFA serial after you have set up everything. The AWS console\n   will not tell you the MFA serial in advance, but by replacing the account id\n   and IAM username, you can build it on your own, e.g.:\n   ```\n   $ ykman oath add -t arn:aws:iam::123456789012:mfa/tommie-lie ABCD1234...\n   ```\n   \n   The above command requires you to touch your YubiKey to generate\n   authentication codes. You can ommit `-t` if you don't want to touch your key\n   every time you authenticate.\n5. Now you have to enter two **consecutive** MFA codes into the AWS website\n   to assign your key to your AWS login. Just run\n   `ykman oath code arn:aws:iam::${ACCOUNT_ID}:mfa/${IAM_USERNAME}`\n   to get an authentication code.\n   The codes are re-generated every 30 seconds, so you have to run this command\n   twice with about 30 seconds in between to get two distinct codes.\n   \n   Enter the two codes in the AWS form and click `Assign MFA`\n6. You're done!\n\n\n### Setup AWS CLI\n\n1. AWS CLI only asks for MFA if you change roles, but role-based access\n   management is a good practice for AWS security anyway, so we'll assume you\n   already have a role and a profile configured in your AWS CLI config file.\n2. Install `awscli-plugin-yubikeytotp`, see [installation section](#installation).\n3. Edit your AWS CLI config file (`~/.aws/config`) and add the MFA serial to\n   the profile you want to use MFA authentication with:\n   <pre><code>\n   [profile yubikey]\n   role_arn = arn:aws:iam::123456789012:role/yubikey-role\n   <b>mfa_serial = arn:aws:iam::123456789012:mfa/tommie-lie</b>\n   source_profile = default\n   </code></pre>\n4. Enable the plugin by appending the following section to your AWS CLI config file:\n   ```\n   [plugins]\n   yubikeytotp = awscli_plugin_yubikeytotp\n   ```\n5. That's it, you're ready to use AWS CLI with your YubiKey now!\n\n\n### Test your setup\n\nIf you need authentication by switching to another IAM role to access certain\nresources, you will normally get a permission denied error:\n```\n$ aws s3 ls\n\nAn error occurred (AccessDenied) when calling the ListBuckets operation: Access Denied\n```\nIf you switch profiles, however, AWS CLI will automatically ask for an MFA code:\n```\n$ aws s3 ls --profile yubikey\nGenerating OATH code on YubiKey. You may have to touch your YubiKey to proceed...\nSuccessfully created OATH code.\n2013-07-11 17:08:50 mybucket\n2013-07-24 14:55:44 mybucket2\n```\n\n\n## Acknowledgements\n* Thanks to [@woowa-hsw0](https://github.com/woowa-hsw0) for this\n  [inspiration for this plugin](https://gist.github.com/woowa-hsw0/caa3340e2a7b390dbde81894f73e379d)\n",
    'author': 'Thomas Liebetraut',
    'author_email': 'thomas@tommie-lie.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tommie-lie/awscli-plugin-yubikeytotp',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from . import inject_yubikey_totp_prompter
import botocore.session

old_init = botocore.session.Session.__init__


def patched_session_init(self, *args, **kwargs):
    old_init(self, *args, **kwargs)
    inject_yubikey_totp_prompter(self)


botocore.session.Session.__init__ = patched_session_init



from awscli.customizations.commands import BasicCommand
import sys
from . import boto_plugin


class SessionEnv(BasicCommand):
    NAME = "session-env"
    DESCRIPTION = (
        "prints the current session's credentials in the form of "
        "environment variables.\n"
        "\n"
        "You can use the ``--profile`` argument to select a different set "
        "of credentials.\n"
        "\n"
        ".. note::\n"
        "  If you have set the environment variables in your shell, subsequent "
        "  calls to aws session-env will use those credentials. In order to use "
        "  your default profile, you have to explicitly specify it::\n"
        "\n"
        "    $(aws session-env --profile default\n"
    )
    SYNOPSIS = "aws session-env"
    EXAMPLES = (
        "Print temporary session tokens for a given profile::\n"
        "\n"
        "  $ aws session-env --profile profile_name\n"
        "  export AWS_ACCESS_KEY_ID=AKIAI44QH8DHBEXAMPLE\n"
        "  export AWS_SECRET_ACCESS_KEY=wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY\n"
        "  export AWS_SESSION_TOKEN=AQoDYXdzEJr...\n"
        "  The acquired credentials will be valid for 60:00 minutes\n"
        "\n"
        "Directly set the environment variables for use in other applications::\n"
        "\n"
        "  $ $(aws session-env --profile profile_name\n"
        "  The acquired credentials will be valid for 60:00 minutes\n"
    )

    ARG_TABLE = []

    def _run_main(self, args, parsed_globals):
        credentials = self._session.get_credentials()

        frozen_credentials = credentials.get_frozen_credentials()
        print("export AWS_ACCESS_KEY_ID={}".format(frozen_credentials.access_key))
        print("export AWS_SECRET_ACCESS_KEY={}".format(frozen_credentials.secret_key))
        if frozen_credentials.token is None:
            print("unset AWS_SESSION_TOKEN")
        else:
            print("export AWS_SESSION_TOKEN={}".format(frozen_credentials.token))

        if hasattr(credentials, "_seconds_remaining"):
            seconds_to_expire = int(credentials._seconds_remaining())
            print(
                "The acquired credentials will be valid for {:.0f}:{:02.0f} minutes".format(
                    seconds_to_expire // 60, seconds_to_expire % 60
                ),
                file=sys.stderr,
            )
        return 0
